;;; prim-kanten.scm

; Das Programm benoetigt die ausgegliederten Funktionen:
(load "prioritaetswarteschlange.scm")
; mit der Funktion (fuege-alle-ein <kantenliste> vor? <priows>),
; die intern die Funktion (fuege-eine-ein <kante> vor? <priows>) verwendet
; Das Praedikat vor? muss dafuer definiert werden:
(define
  (vor? kante-1 kante-2)
  (< (third kante-1) (third kante-2)))
; und
(load "gallenbacher-graph-kanten.scm")
; Verwendet wird davon die Liste "kanten"
; verwenden Sie einen anderen Graphen muss auch vorhanden sein:
; die Funktion (nachfolge-kanten <aktuelle-stadt> <kanten>)

;;; =============== Entwurfsprinzip fuer Prim ===============
;;; Zum Startknoten, danach jeweils dem aktuellen weiter
;;; fuehrenden Knoten werden die Nachfolgekanten
;;; in die Prioritaetswarteschlange (priows) eingefuegt
;;; bis alle Knoten besucht worden sind

;;; ===== prim =====
; mit einer passenden Aufrufhuelle
(define
  (prim start-knoten kanten)
  (define
    (prim-intern besucht anzahl kuerzeste priows)
    (cond
      ((= (length besucht) anzahl) ; alle Knoten enthalten
       kuerzeste)
      ((null? priows)
       #f)
      ((zyklus? (first priows) besucht) ; freier Knoten in besucht?
       (prim-intern besucht anzahl kuerzeste (rest priows)))
      (else
       (prim-intern
        (aktualisiere-besuchtliste (second (first priows)) besucht)
        anzahl
        (cons (first priows) kuerzeste)
        (aktualisiere-priows priows)))
      )) ; Ende interne Funktion
  ; interner Aufruf:
  (prim-intern
   (list start-knoten)
   (anzahl-knoten kanten)
   '()
   (fuege-alle-ein ; Start-PrioWS
    (nachfolge-kanten start-knoten kanten)
    vor?
    '())
   ))

;;; ===== zyklus? =====
; prüft, ob der freie Knoten der Kante in der Besuchtliste enthalten ist
(define (zyklus? kante besucht)
   (member (second kante) besucht))

;;; ===== Hilfsfunktion aktualisiere-besuchtliste =====
(define
  (aktualisiere-besuchtliste knoten besucht)
  (cond
   ((member knoten besucht)
    besucht)
   (else
    (cons knoten besucht))
   ))
;; ===== Hilfsfunktion aktualisiere-priows =====
; Zur ersten Kante aus der Prioritaetswarteschlange werden alle weiter fuehrenden
; Kanten bestimmt und in den Rest der priows eingefuegt
(define
  (aktualisiere-priows priows)
  (fuege-alle-ein
   (nachfolge-kanten (second (first priows)) kanten)
   vor?
   (rest priows)))

;;; ===== Hilfsfunktion anzahl-knoten =====
; Voraussetzung: Die Liste kanten enthaelt jeweils
; auch die Rueckkante
(define
  (anzahl-knoten kanten)
  (define
    (anzahl-intern kanten gefunden)
    ; arbeitet endrekursiv
    (cond
      ((null? kanten) (length gefunden))
      ((member (first (first kanten)) gefunden)
       (anzahl-intern (rest kanten) gefunden))
      (else
       (anzahl-intern
        (rest kanten)
        (cons (first (first kanten)) gefunden)))))
  (anzahl-intern kanten '()))

; Tests:
(prim 'I kanten)
;(prim 'E kanten)

;;; ===== Zusatzfunktion laenge-netz =====
(define
  (laenge-netz netz-kanten)
  (apply + (map third netz-kanten)))

; Tests:
(laenge-netz (prim 'I kanten)) 
;(laenge-netz (prim 'E kanten)) 

